from PySide6.QtWidgets import QWidget


class MplCanvas(QWidget):
    """在qt中使用matplotlib绘制图像"""

    def __init__(self, width: float = 5, height: float = 4, dpi: int = 100, enable_toolbar: bool = True, parent=None):
        """
        :param width: 图像宽度
        :param height: 图像高度
        :param dpi: 图像dpi
        :param enable_toolbar: 是否启用顶部工具栏
        :param parent: 所属的ui父类
        """
        from matplotlib import pyplot as plt
        from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas, \
            NavigationToolbar2QT as NavigationToolbar
        from matplotlib.figure import Figure
        from PySide6.QtWidgets import QVBoxLayout
        plt.rcParams['font.sans-serif'] = ['SimHei']  # 用来正常显示中文标签
        plt.rcParams['axes.unicode_minus'] = False  # 用来正常显示负号

        super(MplCanvas, self).__init__(parent)
        self.canvas = FigureCanvas(Figure(figsize=(width, height), dpi=dpi, constrained_layout=True))
        self.ax = self.canvas.figure.subplots()

        self.vertical_layout = QVBoxLayout()
        if enable_toolbar:
            self.toolbar = NavigationToolbar(self.canvas, self)
            self.vertical_layout.addWidget(self.toolbar)
        self.vertical_layout.addWidget(self.canvas)
        self.setLayout(self.vertical_layout)

    def clear_canvas(self):
        """清空画布"""
        self.canvas.figure.clear()
        self.ax = self.canvas.figure.subplots()
        self.canvas.draw_idle()

    def plot_lines(self, data: list[dict],
                   x_label: str = '',
                   y_label: str = '',
                   title: str = '',
                   marker='o',
                   markersize=3,
                   *args, **kwargs):
        """绘制点线图
        :param data: 每个点的数据使用字典存储，结构为{'x': , 'y': , 'label': }
        :param x_label: x轴名称
        :param y_label: y轴名称
        :param title: 图像名称
        :param marker: 数据点样式
        :param markersize: 数据点大小
        :param args: ax.plot函数接收的其他参数
        :param kwargs: ax.plot函数接收的其他参数
        """
        enable_legend = False
        for item in data:
            if item.get('label'):
                enable_legend = True
                self.ax.plot(item['x'], item['y'], label=item['label'], marker=marker, markersize=markersize, *args, **kwargs)
            else:
                self.ax.plot(item['x'], item['y'], marker=marker, markersize=markersize, *args, **kwargs)

        # 设置轴标签和标题
        self.ax.set_xlabel(x_label)
        self.ax.set_ylabel(y_label)
        self.ax.set_title(title)
        if enable_legend:
            self.ax.legend(loc="upper left", framealpha=0.6)
        self.canvas.draw_idle()

    def plot_bars(self,
                  data: list[dict],
                  x_label: str = '',
                  y_label: str = '',
                  title: str = "",
                  width: float = 0.5,
                  alpha=0.9,
                  *args, **kwargs):
        """绘制多组对比直方图
        :param data: 每个点的数据使用字典存储，结构为{'x': , 'y': , 'label': }
        :param x_label: x轴名称
        :param y_label: y轴名称
        :param title: 图像名称
        :param width: 直方图宽度
        :param alpha: 直方图透明度
        :param args: ax.bar函数接收的其他参数
        :param kwargs: ax.bar函数接收的其他参数
        """
        n = len(data)
        enable_legend = False
        for i, item in enumerate(data):
            if item.get('label'):
                enable_legend = True
            offset = width / 2 * (n - 2 * i - 1)
            x_list = [x - offset for x in item['x']]
            y_list = item['y']
            self.ax.bar(x_list, y_list, width=width, label=item['label'], alpha=alpha, *args, **kwargs)

        # 设置轴标签和标题
        self.ax.set_xlabel(x_label)
        self.ax.set_ylabel(y_label)
        self.ax.set_title(title)
        if enable_legend:
            self.ax.legend(loc="upper left", framealpha=0.6)
        self.canvas.draw_idle()



from PySide6.QtCore import QObject, Signal, QFile
from PySide6.QtUiTools import QUiLoader
from PySide6.QtWidgets import QVBoxLayout, QHBoxLayout

loader = QUiLoader()


def load_ui(file_path: str, parent=None):
    """动态加载ui文件"""
    file = QFile(file_path)
    file.open(QFile.ReadOnly)
    ui = loader.load(file, parent)
    file.close()
    return ui


def set_ui(ui, parent, layout: str = 'v'):
    """将单个或多个ui类添加到parent中。
    :param ui: 单个UI组件（QWidget或其子类实例），或者UI组件的可迭代对象（如列表、元组）。
    :param parent: 父QWidget，传入的UI组件将作为子组件添加到这个父组件中。
    :param layout: 设置布局的方向为竖直'v'或者水平'h'
    """
    layout = QVBoxLayout(parent) if layout == 'v' else QHBoxLayout(parent)
    if isinstance(ui, (list, tuple)):  # 检查ui是否是列表或元组
        for ui_item in ui:  # 遍历UI组件的可迭代对象
            layout.addWidget(ui_item)  # 将每个UI组件添加到布局中
    else:  # ui不是可迭代对象，即单个UI组件
        layout.addWidget(ui)  # 直接将UI组件添加到布局中
    layout.setContentsMargins(0, 0, 0, 0)
    layout.setSpacing(0)
    parent.setLayout(layout)


def force_draw():
    """强制绘制挂起的事件，如弹窗，如果一次调用仍然存在未绘制的情况，可以多调用几次，在show前show后均调用"""
    from PySide6.QtWidgets import QApplication
    QApplication.processEvents()


def create_signal(signal_type):
    """根据传入类型，创建相应的信号类。
    :param signal_type: 任意的数据类型，如int, str, list等
    :return: qt信号类
    """

    class CustomSignal(QObject):
        signal = Signal(signal_type)

    return CustomSignal()


class ProgressSignalSlot:
    """需要某个类继承它，并连接response_signal方法到tuple类型的信号。
    然后根据需要实现各种when_xxx方法，即可根据不同状态自动调用相应的处理方法。"""

    def response_signal(self, state: tuple):
        """内置方法，直接连接外部的qt信号"""
        flag, data = state
        if flag == Progress.flag_start:
            self.when_start(data)
        elif flag == Progress.flag_doing:
            self.when_doing(data)
        elif flag == Progress.flag_success:
            self.when_success(data)
        elif flag == Progress.flag_failed:
            self.when_failed(data)
        elif flag == Progress.flag_end:
            self.when_end(data)
        elif flag == Progress.flag_info:
            self.when_info(data)
        elif flag == Progress.flag_error:
            self.when_error(data)
        elif flag == Progress.flag_other1:
            self.when_other1(data)
        elif flag == Progress.flag_other2:
            self.when_other2(data)
        elif flag == Progress.flag_other3:
            self.when_other3(data)
        else:
            raise ValueError(f'传递的信号数据错误。应该是 tuple ，但得到的是 {type(state)}')

    def when_start(self, data: list = None):
        """事件刚开始
        :param data: 当需要传输数据时，使用此参数
        """
        pass

    def when_doing(self, data: list = None):
        """事件正在进行中
        :param data: 当需要传输数据时，使用此参数
        """
        pass

    def when_success(self, data: list = None):
        """事件成功
        :param data: 当需要传输数据时，使用此参数
        """
        pass

    def when_failed(self, data: list = None):
        """事件失败
        :param data: 当需要传输数据时，使用此参数
        """
        pass

    def when_end(self, data: list = None):
        """事件结束
        :param data: 当需要传输数据时，使用此参数
        """
        pass

    def when_info(self, data: list = None):
        """事件信息
        :param data: 当需要传输数据时，使用此参数
        """
        pass

    def when_error(self, data: list = None):
        """事件错误
        :param data: 当需要传输数据时，使用此参数
        """
        pass

    def when_other1(self, data: list = None):
        """预留的任意状态
        :param data: 当需要传输数据时，使用此参数
        """
        pass

    def when_other2(self, data: list = None):
        """预留的任意状态
        :param data: 当需要传输数据时，使用此参数
        """
        pass

    def when_other3(self, data: list = None):
        """预留的任意状态
        :param data: 当需要传输数据时，使用此参数
        """
        pass


class Progress:
    """搭配ProgressSignalSlot使用，发射信号时应该使用此类来传输数据。

    Example:
        # 具体数据应使用list包裹，这利于处理复杂形态的数据。
        some_signal.emit(Progress.start(['处理开始']))

    """
    flag_start = 0
    flag_doing = 1
    flag_end = 2
    flag_success = 3
    flag_failed = 4
    flag_info = 5
    flag_error = 6
    # 预留的其他状态量
    flag_other1 = 7
    flag_other2 = 8
    flag_other3 = 9

    @staticmethod
    def start(data: list = None) -> tuple:
        return Progress.flag_start, data

    @staticmethod
    def doing(data: list = None) -> tuple:
        return Progress.flag_doing, data

    @staticmethod
    def end(data: list = None) -> tuple:
        return Progress.flag_end, data

    @staticmethod
    def success(data: list = None) -> tuple:
        return Progress.flag_success, data

    @staticmethod
    def failed(data: list = None) -> tuple:
        return Progress.flag_failed, data

    @staticmethod
    def info(data: list = None) -> tuple:
        return Progress.flag_info, data

    @staticmethod
    def error(data: list = None) -> tuple:
        return Progress.flag_error, data

    @staticmethod
    def other1(data: list = None) -> tuple:
        return Progress.flag_other1, data

    @staticmethod
    def other2(data: list = None) -> tuple:
        return Progress.flag_other2, data

    @staticmethod
    def other3(data: list = None) -> tuple:
        return Progress.flag_other3, data



# -*- coding:utf-8 -*-
from collections import OrderedDict

from PySide6.QtCore import QItemSelectionModel, Qt, QObject, QModelIndex, QEvent
from PySide6.QtGui import QStandardItem, QColor, QStandardItemModel

from qtlink.util import create_signal


class TableController:
    """QTableView的基础控制类，负责存储/显示/交互表格数据。"""

    def __init__(self, tableview, highlight_hover_row: bool = False):
        """
        :param tableview: QTableView，需要显示表格数据的ui对象。
        :param highlight_hover_row: 是否启用鼠标悬停时高亮一整行的样式，默认不启用。
        """
        self.tableview = tableview
        self.table_columns = []
        self.raw_data = None
        self.column_widths = None
        self.first_load = True
        self.model = QStandardItemModel()
        # 点击某行时，会用qt信号发送该行的字典格式数据。如果需要处理这类数据，则应连接该信号到自定义的槽函数。
        self.signal_click_row = create_signal(dict)
        # 连接信号
        self.tableview.clicked.connect(self.on_table_clicked)
        # 禁止上下文菜单
        scrollBar = self.tableview.verticalScrollBar()
        scrollBar.setContextMenuPolicy(Qt.NoContextMenu)
        scrollBar = self.tableview.horizontalScrollBar()
        scrollBar.setContextMenuPolicy(Qt.NoContextMenu)
        # 启用高亮一整行效果
        if highlight_hover_row:
            self.tableview.setMouseTracking(True)
            self.hover_delegate = HoverDelegate(tableview=tableview)
            self.tableview.entered.connect(self.hover_delegate.on_entered)
        else:
            self.hover_delegate = None

    def update_table_data(self, table_data: list[dict] = None,
                          hide_columns: list[str] = None,
                          init_columns: list[str] = None):
        """更新表格数据
        :param table_data: 原始表格数据，表格中的每一行数据都应使用字典格式，其中的键将按顺序自动识别为表格的列名。
        :param hide_columns: 需要隐藏/不显示的列名。
        :param init_columns: 用于无数据时初始化显示表头。
        """
        if table_data is None:
            table_data = []
        self.before_update_table_data(table_data, hide_columns=hide_columns, init_columns=init_columns)

        for data in table_data:
            items = [QStandardItem(str(data.get(column, None)) if data.get(column, None) is not None else '')
                     for column in self.table_columns]
            self.model.appendRow(items)

        self.set_table()

    def before_update_table_data(self,
                                 table_data: list[dict],
                                 hide_columns: list[str] = None,
                                 init_columns: list[str] = None):
        if not self.first_load:
            # 保存当前列宽
            self.save_current_column_widths()
        self.first_load = False
        self.raw_data = table_data
        # 传入空列表将同步清空表头，传入只有键无值的结构才保持显示表头，而无表格数据。

        if table_data:
            self.table_columns = self.get_table_columns_from_data(table_data, hide_columns=hide_columns)
        elif init_columns:
            self.table_columns = init_columns
        elif self.table_columns:
            pass
        else:
            raise ValueError('表头设置错误')
        # self.model.clear()
        del self.model
        self.model = QStandardItemModel()

    def set_table(self):
        self.model.setHorizontalHeaderLabels(self.table_columns)
        self.tableview.setModel(self.model)

        if self.column_widths is not None:
            for item in self.column_widths:
                index, width = item
                self.tableview.setColumnWidth(index, width)

    def get_table_columns_from_data(self, data: list[dict], hide_columns: list[str] = None):
        ordered_keys = OrderedDict()
        for d in data:
            for key in d.keys():
                if hide_columns and key in hide_columns:
                    continue
                ordered_keys[key] = None
        # 将OrderedDict的键转换为列表
        return list(ordered_keys.keys())

    def clear_table_data(self):
        """清空表格数据"""
        self.update_table_data(table_data=[])

    def save_current_column_widths(self):
        if self.column_widths is None:
            return
        for index in range(self.model.columnCount()):
            width = self.tableview.columnWidth(index)
            # 确保保存的列宽信息与列的数量相匹配
            if index < len(self.column_widths):
                self.column_widths[index] = (index, width)
            else:
                self.column_widths.append((index, width))

    def on_table_clicked(self, index):
        # index是QModelIndex类型，可以用来检索数据
        row = index.row()  # 获取行号
        # 获取该行的所有列的数据
        rowData = self.raw_data[row]
        self.signal_click_row.signal.emit(rowData)

    def set_column_width(self, column_widths: list[tuple] = None):
        """通过多个(index, width)元组指定特定列的宽度。
        注意：应该在调用update_table_data之前完成设置。
        :param column_widths: 任意个(index, width)组成的list
        """
        self.column_widths = column_widths

    def set_select_row(self, index: int):
        if len(self.raw_data) - 1 >= index:
            first_index = self.model.index(index, 0)
            self.tableview.selectionModel().select(first_index,
                                                   QItemSelectionModel.Select | QItemSelectionModel.Rows)

    def set_not_select_row(self, index: int):
        if len(self.raw_data) - 1 >= index:
            first_index = self.model.index(index, 0)
            self.tableview.selectionModel().select(first_index,
                                                   QItemSelectionModel.Deselect | QItemSelectionModel.Rows)

    def __del__(self):
        if self.hover_delegate and self.tableview.viewport():
            self.tableview.viewport().removeEventFilter(self.hover_delegate)


class HoverDelegate(QObject):
    def __init__(self, tableview):
        super().__init__(tableview)
        self.tableview = tableview
        self.row_under_mouse = -1
        self.hover_color = QColor(240, 240, 240)
        # 安装事件过滤器到表格视图的视口上
        self.tableview.viewport().installEventFilter(self)
        self.is_hovering = False

    def on_entered(self, index):
        self.is_hovering = True
        hover_row = index.row()
        # 检查当前行是否被选中
        is_row_selected = self.tableview.selectionModel().isRowSelected(hover_row, QModelIndex())
        if hover_row != self.row_under_mouse:
            # 还原上一行的背景色
            if self.row_under_mouse >= 0:
                self.set_row_background(self.row_under_mouse, self.tableview.palette().base())
            # 重设当前行背景色
            if not is_row_selected:
                self.set_row_background(hover_row, self.hover_color)
                self.row_under_mouse = hover_row
        self.is_hovering = False

    def set_row_background(self, hover_row, color):
        for column in range(self.tableview.model().columnCount()):
            index = self.tableview.model().index(hover_row, column)
            rect = self.tableview.visualRect(index)
            self.tableview.viewport().update(rect)
            self.tableview.model().setData(index, color, Qt.BackgroundRole)

    def clear_hover_style(self):
        if self.row_under_mouse >= 0:
            self.set_row_background(self.row_under_mouse, self.tableview.palette().base())
            self.row_under_mouse = -1
            self.tableview.viewport().update()  # 更新视图以反映背景色的变化

    def eventFilter(self, watched, event):
        if watched and watched == self.tableview.viewport() and event.type() == QEvent.Leave:
            self.clear_hover_style()
        return super().eventFilter(watched, event)



from PySide6.QtWidgets import QDialog, QLabel
from PySide6.QtWidgets import QLineEdit, QPushButton, QVBoxLayout, QHBoxLayout

from qtlink import create_signal


class DialogInput(QDialog):
    """带有单行输入框的对话框"""
    def __init__(self,
                 text='',
                 placeholder='',
                 place_text='',
                 title='提示',
                 parent=None):
        """
        :param text: 对话框的文本消息，为''时会自动隐藏
        :param placeholder: 输入框的提示语
        :param place_text: 输入框的默认文字
        :param title: 设置窗口标题
        :param parent: 所属的父类
        """
        super().__init__(parent=parent)
        # 数据信号，用于获取输入内容
        self.signal_data = create_signal(str)
        self.setWindowTitle(title)

        label = QLabel(text, self)
        label.setWordWrap(True)
        label.setText(text)

        # 创建 QLineEdit
        self.lineEdit = QLineEdit(self)

        # 创建确定按钮和取消按钮
        self.okButton = QPushButton("确定", self)
        self.cancelButton = QPushButton("取消", self)

        self.lineEdit.setPlaceholderText(placeholder)
        if place_text:
            self.lineEdit.setText(place_text)
        # 设置按钮点击事件处理函数
        self.okButton.clicked.connect(self.click_ok)
        self.cancelButton.clicked.connect(self.reject)

        # 按钮水平布局
        h_layout = QHBoxLayout()
        h_layout.addWidget(self.okButton)
        h_layout.addWidget(self.cancelButton)

        # 总体垂直布局
        layout = QVBoxLayout(self)
        if text:
            layout.addWidget(label)
        layout.addWidget(self.lineEdit)
        layout.addLayout(h_layout)  # 将水平布局加入垂直布局

        self.setLayout(layout)

    def click_ok(self):
        # 点击确定按钮的事件处理
        self.accept()
        self.signal_data.signal.emit(self.lineEdit.text())



from PySide6.QtWidgets import QDialog, QTableView, QLabel, QVBoxLayout, QAbstractItemView

from qtlink.table.table_controller import TableController


class DialogTableDisplay(QDialog):
    """可以显示表格数据的对话框"""

    def __init__(self, text: str,
                 table_data: list[dict],
                 parent=None,
                 title: str = '提示',
                 *args, **kwargs):
        """
        :param text: 对话框的文本消息
        :param table_data: 表格数据
        :param parent: 对话框所属的父类
        :param args: 表格控制器可能使用的其他参数
        :param kwargs: 表格控制器可能使用的其他参数
        """
        super().__init__(parent=parent)
        self.v_layout = QVBoxLayout()
        label = QLabel(text, self)
        label.setWordWrap(True)
        self.tableview = QTableView(self)
        self.tableview.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.v_layout.addWidget(label)
        self.v_layout.addWidget(self.tableview)

        self.setWindowTitle(title)
        self.setLayout(self.v_layout)
        self.table_controller = TableController(tableview=self.tableview)
        self.table_controller.update_table_data(table_data=table_data, *args, **kwargs)



from typing import Optional, Callable, Iterable

from PySide6.QtWidgets import QDialog, QVBoxLayout, QLabel, QPushButton, QHBoxLayout


class DialogInfo(QDialog):
    """显示文本消息和简单交互按钮的对话框类"""

    def __init__(self,
                 text: str,
                 title: str = '提示',
                 set_buttons: Optional[Iterable[tuple[str, Optional[Callable]]]] = None,
                 parent=None):
        """
        :param text: 对话框的文本消息
        :param title: 设置窗口标题
        :param set_buttons: 设置按钮参数，str指按钮文本，callable指点击按钮后调用的函数。如[('确定', None), ('取消', None)]
        :param parent: 所属的父类
        """
        super().__init__(parent=parent)

        self.set_buttons = set_buttons
        self.setWindowTitle(title)

        vLayout = QVBoxLayout()
        label = QLabel(text, self)
        label.setWordWrap(True)
        vLayout.addWidget(label)

        if self.set_buttons:
            h_layout = QHBoxLayout()
            for btn in self.set_buttons:
                button = QPushButton(btn[0], self)
                button.clicked.connect(self.create_button_slot(btn[1]))
                h_layout.addWidget(button)
            vLayout.addLayout(h_layout)

        self.setLayout(vLayout)

    def create_button_slot(self, func):
        def button_slot():
            if func:
                func()
            self.close()
        return button_slot



import setuptools

setuptools.setup(
    name="qtlink",
    version="1.2.17",
    author="NanHaiLoong",
    author_email="nanhai@163.com",
    description="a ui framework based on pyside6",
    long_description='a ui framework based on pyside6',
    long_description_content_type="text/markdown",
    url="https://gitee.com/darlingxyz/qtlink",
    install_requires=['PySide6'],
    packages=setuptools.find_packages(include=['qtlink', 'qtlink.*']),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



# -*- coding:utf-8 -*-
from PySide6.QtCore import Qt
from PySide6.QtGui import QStandardItem

from qtlink.table.table_controller import TableController
from qtlink.util import create_signal


class TableControllerSingleCheck(TableController):
    """QTableView的单选控制类，它继承自基本控制类，因此它也有基本控制类的属性和方法。"""

    def __init__(self, tableview, highlight_hover_row: bool = False):
        """
        :param tableview: QTableView，需要显示表格数据的ui对象。
        :param highlight_hover_row: 是否启用鼠标悬停时高亮一整行的样式，默认不启用。
        """
        super().__init__(tableview, highlight_hover_row)
        self.model.itemChanged.connect(self.on_item_changed)
        # 当勾选某一行时，会触发该信号发送该行的字典格式数据。如果需要处理这类数据，则应连接该信号到自定义的槽函数。
        self.signal_checked_rows = create_signal(list)

    def update_table_data(self, table_data: list[dict] = None,
                          hide_columns: list[str] = None,
                          init_columns: list[str] = None):
        # 断开信号连接
        self.model.itemChanged.disconnect(self.on_item_changed)
        if table_data is None:
            table_data = []
        self.before_update_table_data(table_data, hide_columns=hide_columns, init_columns=init_columns)
        for data in table_data:
            items = [QStandardItem(str(data.get(column, None)) if data.get(column, None) is not None else '')
                     for column in self.table_columns]
            items[0].setCheckable(True)
            self.model.appendRow(items)

        self.set_table()
        # 重新连接信号
        self.model.itemChanged.connect(self.on_item_changed)
        self.on_item_changed()

    def on_item_changed(self, item: QStandardItem = None):
        """只管理数据的发送"""
        if item is None:
            return
        if item.checkState() == Qt.Checked:
            self.model.itemChanged.disconnect(self.on_item_changed)
            for row in range(self.model.rowCount()):
                if self.model.item(row, 0) != item:
                    self.model.item(row, 0).setCheckState(Qt.Unchecked)
            self.model.itemChanged.connect(self.on_item_changed)

        checked_rows = self.get_checked_rows()
        self.signal_checked_rows.signal.emit(checked_rows)

    def get_checked_rows(self) -> list[dict]:
        """返回被选中行的字典数据。为了与多选表格控制类保持统一格式，该方法返回的是列表格式。"""
        checked_rows = []
        for row in range(self.model.rowCount()):
            item = self.model.item(row, 0)  # 勾选框在第一列
            if item.checkState() == Qt.Checked:
                checked_rows.append(self.raw_data[row])
                break
        return checked_rows



# -*- coding:utf-8 -*-
from PySide6.QtCore import Qt, QRect
from PySide6.QtGui import QStandardItem
from PySide6.QtWidgets import QStyleOptionButton, QStyle, QHeaderView

from qtlink.table.table_controller import TableController
from qtlink.util import create_signal


class TableControllerMultipleCheck(TableController):
    """QTableView的多选控制类，它继承自基本控制类，所以基本控制类有的参数和方法它都有。"""

    def __init__(self, tableview, highlight_hover_row: bool = False):
        """
        :param tableview: QTableView，需要显示表格数据的ui对象。
        :param highlight_hover_row: 是否启用鼠标悬停时高亮一整行的样式，默认不启用。
        """
        super().__init__(tableview, highlight_hover_row)
        self.model.itemChanged.connect(self.on_item_changed)
        # 当勾选若干行时，会触发该信号发送这些行的字典格式数据。如果需要处理这类数据，则应连接该信号到自定义的槽函数。
        self.signal_checked_rows = create_signal(list)
        self.header = CustomHeader(toggle_check_state=self.toggle_check_state, parent=self.tableview)
        self.tableview.setHorizontalHeader(self.header)
        self.always_checked_data = []

    def update_table_data(self, table_data: list[dict] = None,
                          hide_columns: list[str] = None,
                          default_checked_table_data: list[dict] = None,
                          always_checked_table_data: list[dict] = None,
                          init_columns: list[str] = None):
        """更新表格数据
        :param table_data: 原始表格数据，表格中的每一行数据都应使用字典格式，其中的键将按顺序自动识别为表格的列名。
        :param hide_columns: 需要隐藏/不显示的列名。
        :param default_checked_table_data: 默认被选中的数据。
        :param always_checked_table_data: 始终被选中的数据。
        :param init_columns: 用于无数据时初始化显示表头。
        """
        # 断开信号连接
        self.model.itemChanged.disconnect(self.on_item_changed)

        if table_data is None:
            table_data = []
        self.before_update_table_data(table_data, hide_columns=hide_columns, init_columns=init_columns)
        if default_checked_table_data is None:
            default_checked_table_data = []
        self.always_checked_data = always_checked_table_data if always_checked_table_data else []

        for data in table_data:
            items = [QStandardItem(str(data.get(column, None)) if data.get(column, None) is not None else '')
                     for column in self.table_columns]
            items[0].setCheckable(True)
            # 默认被选中
            if data in default_checked_table_data:
                items[0].setCheckState(Qt.Checked)
            # 始终被选中
            if data in self.always_checked_data:
                items[0].setCheckState(Qt.Checked)
                # 禁用交互
                for item in items:
                    item.setFlags(Qt.ItemIsUserCheckable | Qt.ItemIsSelectable)
            self.model.appendRow(items)

        self.set_table()
        # 重新连接信号
        self.model.itemChanged.connect(self.on_item_changed)
        self.on_item_changed()

    def toggle_check_state(self, is_checked):
        # 断开信号连接
        self.model.itemChanged.disconnect(self.on_item_changed)

        for row in range(self.model.rowCount()):
            item = self.model.item(row)
            # 重设勾选状态
            if self.raw_data[row] not in self.always_checked_data:
                item.setCheckState(Qt.Checked if is_checked else Qt.Unchecked)

        # 重新连接信号
        self.model.itemChanged.connect(self.on_item_changed)
        self.on_item_changed()

    def get_checked_rows(self) -> list[dict]:
        """返回被选中的若干行的字典数据。为了与单选表格控制类保持统一格式，该方法返回的是列表格式。"""
        checked_rows = []
        for row in range(self.model.rowCount()):
            item = self.model.item(row, 0)  # 勾选框在第一列
            if item.checkState() == Qt.Checked:
                checked_rows.append(self.raw_data[row])
        return checked_rows

    def on_item_changed(self, item: QStandardItem = None):
        if self.hover_delegate is not None and self.hover_delegate.is_hovering:
            return

        checked_rows = self.get_checked_rows()
        self.signal_checked_rows.signal.emit(checked_rows)
        checked_rows_num = len(checked_rows)
        if checked_rows_num and checked_rows_num == len(self.raw_data):
            self.set_header_check_state(True)
        else:
            self.set_header_check_state(False)

    def set_header_check_state(self, is_checked: bool):
        self.header.update_check_state(is_checked)


class CustomHeader(QHeaderView):
    def __init__(self, toggle_check_state, orientation=Qt.Horizontal, parent=None):
        super(CustomHeader, self).__init__(orientation, parent)
        self.isChecked = False  # 初始化复选框状态为未选中
        self.toggle_check_state = toggle_check_state

    def update_check_state(self, is_checked):
        self.isChecked = is_checked
        self.updateSection(0)

    def mouseReleaseEvent(self, event):
        # 处理鼠标释放事件，用于切换表头复选框的状态
        index = self.logicalIndexAt(event.position().toPoint())
        if index == 0:
            self.update_check_state(not self.isChecked)  # 切换复选框状态
            self.toggle_check_state(self.isChecked)  # 更新模型中所有项的复选框状态
        else:
            super(CustomHeader, self).mouseReleaseEvent(event)

    def paintSection(self, painter, rect, logicalIndex):
        # 绘制表头的每一部分，用于绘制复选框
        painter.save()
        super(CustomHeader, self).paintSection(painter, rect, logicalIndex)
        painter.restore()
        # 如果是第一列，绘制复选框
        if logicalIndex == 0:
            option = QStyleOptionButton()
            checkbox_width = 10  # 复选框宽度
            checkbox_height = 10  # 复选框高度
            checkbox_x = rect.x() + 3  # 复选框的水平位置
            checkbox_y = rect.y() + (rect.height() - checkbox_height) / 2  # 复选框的垂直位置
            option.rect = QRect(checkbox_x, checkbox_y, checkbox_width, checkbox_height)
            option.state = QStyle.State_Enabled | QStyle.State_Active
            if self.isChecked:
                option.state |= QStyle.State_On  # 设置复选框为选中状态
            else:
                option.state |= QStyle.State_Off  # 设置复选框为未选中状态
            self.style().drawControl(QStyle.CE_CheckBox, option, painter)  # 绘制复选框



# -*- coding:utf-8 -*-
from PySide6.QtWidgets import QWidget, QListWidget, QVBoxLayout, QListWidgetItem, \
    QAbstractItemView

from qtlink import set_ui

scrollbar_style = """
QScrollBar:vertical {
    border: none;
    background: #F1F1F1;
    width: 12px;
    margin: 0px 0px 0px 0px;
}

QScrollBar::handle:vertical {
    background: #CCCCCC;
    min-height: 20px;
}

QScrollBar:horizontal {
    border: none;
    background: #F1F1F1;
    height: 12px;
    margin: 0px 0px 0px 0px;
}

QScrollBar::handle:horizontal {
    background: #CCCCCC;
    min-width: 20px;
}
"""

list_widget_style = """
QListWidget::item:hover {
    background-color: rgb(240, 240, 240);
    border: none;
    outline: none;
}

QListWidget::item:selected {
    background-color: rgb(220, 220, 220);
    border: none;
    outline: none;
}

QListWidget:focus {
    outline: none;
}

"""


class ListWidget(QWidget):
    """只是容器"""

    def __init__(self):
        super().__init__()
        # 创建QListWidget
        self.listWidget = QListWidget(self)
        set_ui(self.listWidget, self)
        self.setStyleSheet("QWidget { border: none; }")
        # 应用滚动条样式
        self.listWidget.verticalScrollBar().setStyleSheet(scrollbar_style)
        self.listWidget.horizontalScrollBar().setStyleSheet(scrollbar_style)
        self.listWidget.setStyleSheet(list_widget_style)
        # 设置像素级的垂直滚动
        self.listWidget.setVerticalScrollMode(QAbstractItemView.ScrollPerPixel)  # noqa

    def update_widgets(self, widgets: list):
        self.listWidget.clear()
        for widget in widgets:
            item = QListWidgetItem(self.listWidget)
            item.setSizeHint(widget.sizeHint())
            self.listWidget.addItem(item)
            self.listWidget.setItemWidget(item, widget)



from PySide6.QtWidgets import QDialog, QTableView, QLabel, QVBoxLayout, QAbstractItemView, QPushButton

from qtlink import create_signal
from qtlink.table.table_controller import TableController
from qtlink.table.table_controller_multiple_check import TableControllerMultipleCheck
from qtlink.table.table_controller_single_check import TableControllerSingleCheck


class DialogTableCheck(QDialog):
    """可以显示并选择表格数据的对话框"""

    def __init__(self, text: str,
                 table_data: list[dict],
                 check_type: str = 'single',
                 parent=None,
                 title: str = '选择',
                 *args, **kwargs):
        """
        :param text: 对话框的消息文本
        :param table_data: 表格数据
        :param check_type: 表格数据的选择方式，单选：'single'，多选：'multiple'
        :param parent: 对话框所属的父类
        :param args: 表格控制器可能使用的其他参数
        :param kwargs: 表格控制器可能使用的其他参数
        """
        super().__init__(parent=parent)
        self.v_layout = QVBoxLayout()
        label = QLabel(text, self)
        label.setWordWrap(True)
        self.tableview = QTableView(self)
        self.tableview.setEditTriggers(QAbstractItemView.NoEditTriggers)  # noqa
        self.v_layout.addWidget(label)
        self.v_layout.addWidget(self.tableview)

        self.setWindowTitle(title)
        self.setLayout(self.v_layout)
        self.table_controller = TableController(tableview=self.tableview)
        self.btn_ok = QPushButton('确定')
        self.v_layout.addWidget(self.btn_ok)

        self.setWindowTitle(title)
        self.setLayout(self.v_layout)

        if check_type == 'single':
            self.table_controller = TableControllerSingleCheck(tableview=self.tableview)
        elif check_type == 'multiple':
            self.table_controller = TableControllerMultipleCheck(tableview=self.tableview)
        else:
            raise ValueError(f"check_type的值只允许是：'single' 或 'multiple'，但得到的是：{check_type}")
        self.table_controller.update_table_data(table_data, *args, **kwargs)

        self.signal_checked_rows = create_signal(list)
        self.btn_ok.clicked.connect(self.click_btn_ok)

    def click_btn_ok(self):
        checked_data = self.table_controller.get_checked_rows()
        self.signal_checked_rows.signal.emit(checked_data)
        self.close()



# -*- coding:utf-8 -*-
from PySide6.QtCore import Qt
from PySide6.QtGui import QCursor
from PySide6.QtWidgets import QWidget, QVBoxLayout, QLabel, QFileDialog

from qtlink.util import create_signal


class DropWidget(QWidget):
    def __init__(self, label_text: str, parent=None, default_path: str = ""):
        super().__init__(parent)
        self.default_path = default_path
        self.signal_select_path = create_signal(str)
        self.setCursor(QCursor(Qt.PointingHandCursor))  # 设置鼠标样式为指向手势
        self.setAcceptDrops(True)

        self.setMinimumHeight(130)  # 设置最小高度
        self.setMaximumHeight(130)  # 设置最大高度

        self.label = QLabel(label_text, self)
        self.label.setAlignment(Qt.AlignCenter)
        self.label.setStyleSheet("border: 2px dashed #aaa;")
        self.setStyleSheet("QWidget { background-color: rgb(240, 240, 240); }")

        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)  # 设置布局边距为0
        layout.setSpacing(0)  # 设置布局内部组件间的间距为0
        layout.addWidget(self.label)
        self.setLayout(layout)

    def update_default_path(self, path: str):
        self.default_path = path

    def dragEnterEvent(self, event):
        if event.mimeData().hasUrls():
            event.acceptProposedAction()
        else:
            super().dragEnterEvent(event)

    def dropEvent(self, event):
        mimeData = event.mimeData()
        if mimeData.hasUrls():
            url = mimeData.urls()[0]
            path = url.toLocalFile()
            self.signal_select_path.signal.emit(path)
            event.acceptProposedAction()
        else:
            super().dropEvent(event)

    def mouseReleaseEvent(self, event):
        if event.button() == Qt.LeftButton:
            # 打开文件管理器选择文件夹
            path = QFileDialog.getExistingDirectory(self, "选择文件夹", self.default_path)
            if path:
                self.signal_select_path.signal.emit(path)
